thickness = 1.6;                    //Shell thickness
barb_dia = 15.0;                    //Diameter of the hose barb
barb_id = 8;
barb_ridge = 1.2;                   //Amount the ridge on the hose barb overhangs the barb
barb_height = 15;                   //Overall height of the barb
barb_chamfer = 2;                   //Chamfer between barb and body
inside_board_width = 28;
inside_height = 22;
handle_chamfer = 3;
barb_transition_length = 20;
end_standoff_dia = 10;
thermoset_insert_dia = 4;
handle_length = 96;
lip_height = 3;
lip_inset = 0.5;
screw_recess = 1.2;
screw_head_dia = 6;

outside_width = inside_board_width + 2 * thickness;
outside_height = inside_height + 2 * thickness;

$fs=0.1;

module rounded_box(x, z, r){
    hull(){
        hull(){
            translate([x/2-r,0,-z/2+r])
            sphere(r=r, $fs=0.5);
            translate([-x/2+r,0,-z/2+r])
            sphere(r=r, $fs=0.5);
        }
        hull(){
            translate([x/2-r,0,z/2-r])
            sphere(r=r, $fs=0.5);
            translate([-x/2+r,0,z/2-r])
            sphere(r=r, $fs=0.5);
        }
    }
}

module barb(major_dia, barb_overhang, barb_height, height, barb_chamfer){

    cylinder(h=height, d=major_dia, center=false);
    translate([0,0,height - barb_height])
    cylinder(h=barb_height, d1=major_dia+2 * barb_overhang, d2=major_dia, center=false);
    chamfer(major_dia/2, barb_chamfer);
}

module chamfer(major, minor){
    translate([0,0,minor])
    rotate_extrude(angle=360)
    translate([major+minor,0,0])
    rotate([0,0,180])
    difference(){
        square([minor,minor]);
        circle(r=minor);
    }
}

module outside_chamfer(major, minor){
    translate([0,0,minor])
    rotate_extrude(angle=360)
    translate([major-minor,0,0])
    rotate([0,0,270])
    difference(){
        square([minor,minor]);
        circle(r=minor);
    }
}

module linear_chamfer(length, minor){
    translate([-minor,0,minor])
    rotate([-90,0,0])
    linear_extrude(length)
        difference(){
        square([minor, minor]);
        circle(r=minor);
    }
}

module standoff(height, od, id, chamfer){

    difference(){
        cylinder(h=height, d=od);
        cylinder(h=height, d=id);
    }
    chamfer(od/2,chamfer);
}

module board_mounts(){
    pos = [
            [0,3],
            [10, 14.5 + 3],
            [-10, 14.5 + 3],
            [-10, 14.5 + 3 + 17],
            [10, 14.5 + 3 + 17],
            [-10, 14.5 + 3 + 17 + 53.5],
            [10, 14.5 + 3 + 17 + 53.5],
          ];
    for (i = pos){
        echo(i);
        translate([i[0], i[1] - 91, 0])
        standoff(7, 6, 2.4, 2);
    }
}

module encoder_cut(){

    translate([0, -3 - 40.5 - 20,  0])
    cylinder(d = 8, h = thickness * 2, center = true);
    translate([0, -3 - 40.5 - 20 + 6, thickness - 0.8])
    cylinder(d = 3, h = thickness, center = true);

}

module main_body(){
    hull(){
        rounded_box(barb_dia + handle_chamfer * 2 + barb_chamfer * 2, outside_height, 3);
        translate([0,barb_transition_length,0])
        rounded_box(inside_board_width + 2 * thickness, outside_height, 3);
    }
    hull(){
        translate([0,barb_transition_length,0])
        rounded_box(outside_width, outside_height, handle_chamfer);
        translate([0,barb_transition_length + handle_length,0])
        rounded_box(outside_width, outside_height, handle_chamfer);
    }
}

module main_body_inset(inset=0, height_inset = 0){
    hull(){
        translate([0, inset/2, 0])
        rounded_box(barb_dia + handle_chamfer * 2 + barb_chamfer * 2 - inset, outside_height - height_inset, 3);
        translate([0,barb_transition_length,0])
        rounded_box(inside_board_width + 2 * thickness - inset, outside_height - height_inset, 3);
    }
    hull(){
        translate([0,barb_transition_length,0])
        rounded_box(outside_width - inset, outside_height - height_inset, handle_chamfer);
        translate([0, barb_transition_length + handle_length - inset/2,0])
        rounded_box(outside_width - inset, outside_height - height_inset, handle_chamfer);
    }
}

module main_body_cut(){
    main_body_inset(inset = 2 * thickness, height_inset = 2 * thickness);
    /*
    *hull(){
        translate([0, thickness, 0])
        rounded_box(barb_dia + handle_chamfer * 2 + barb_chamfer * 2 - 2 * thickness, outside_height - 2 * thickness, 3);
        translate([0,barb_transition_length,0])
        rounded_box(inside_board_width + 2 * thickness - 2 * thickness, outside_height - 2 * thickness, 3);
    }
    hull(){
        translate([0,barb_transition_length,0])
        rounded_box(outside_width - 2 * thickness, outside_height - 2 * thickness, handle_chamfer);
        translate([0,barb_transition_length + handle_length - thickness,0])
        rounded_box(outside_width- 2 * thickness, outside_height- 2 * thickness, handle_chamfer);
    }
    */
}


module end_lid_mount(drill=thermoset_insert_dia, through=false){
    translate([-end_standoff_dia/2 + outside_width/2, end_standoff_dia + barb_transition_length + handle_length, -outside_height/2]){
        rotate([0,0,180]){
            difference(){
                cylinder(h=outside_height, d=end_standoff_dia);
                translate([0,0,thickness])
                if (!through){
                    cylinder(h=outside_height - thickness, d=drill);
                }else{
                    translate([0, 0, -thickness])
                    cylinder(h=outside_height, d=drill);
                }
                outside_chamfer(end_standoff_dia/2, handle_chamfer);
            }
            translate([-end_standoff_dia/2,0,0])
                difference(){
                    
                    cube([end_standoff_dia, end_standoff_dia, outside_height]);
                    
                    union(){
                    translate([end_standoff_dia,0,0])
                        linear_chamfer(end_standoff_dia, handle_chamfer);
                    rotate([0,90,0])
                        linear_chamfer(end_standoff_dia, handle_chamfer);
                    
                    if (!through){
                        translate([end_standoff_dia/2, 0, thickness])
                         cylinder(h=outside_height - thickness, d=drill);
                    }else{
                        translate([end_standoff_dia/2, 0, -thickness])
                         cylinder(h=outside_height + 2 * thickness, d=drill);
                    }
                }
                }
        }
    }
}

module inner_lid_mount(cutout=false, clearance=0){
    intersection(){
        translate([8, thickness, - handle_chamfer + thickness])
        difference(){
                union(){
                    cylinder(d = 8 + clearance, h = inside_height, center=true);
                    translate([0, -4 + clearance/2, 0])
                        cube([8 + clearance, 8 + clearance, inside_height], true);
                    translate([4 - clearance/2 , 0, 0])
                        cube([8 + clearance, 8 + clearance, inside_height], true);
                }
                if (!cutout){
                    cylinder(d = thermoset_insert_dia, h = inside_height, center=true);
                }else{
                    cylinder(d = 0.1, h = inside_height, center=true);
                }
        }
            main_body();
    }
}

module inner_lid_drill(drill=thermoset_insert_dia){
    translate([8, thickness, 0])
        cylinder(d = drill, h = outside_height, center=true);
}


module handle(){

    inner_lid_mount();
    mirror([1, 0, 0]) inner_lid_mount();

    translate([0, barb_transition_length + handle_length, - outside_height/2 + thickness]){
        board_mounts();
    }

    //+0.1 fudge to get the pieces to join
    translate([0,-3 + 0.1,0])
        rotate([90,0,0])
            difference(){
                barb(barb_dia, barb_ridge, 5, barb_height, barb_chamfer);
                cylinder(d = barb_id, h = barb_height + thickness);
            }

    difference(){
        union(){
            main_body(barb_transition_length, handle_length, outside_height, outside_width);
            end_lid_mount();
            mirror([1,0,0])
            end_lid_mount();
        }
        union(){
            main_body_cut();
            //Cutout for DC jack
            translate([0, barb_transition_length + handle_length + thickness, -13/2 + thickness + 7 + 1.6])
                cube([9 + 2, thickness *2, 11 + 1], true);
            translate([0, 0, handle_chamfer])
                main_body_cut();
            translate([0, 0, -handle_chamfer/2 + outside_height/2])
                cube([100, 300, handle_chamfer], true);
            translate([0, thickness/2, 0])
            rotate([90,0,0])
                cylinder(d = barb_id, h = barb_height + thickness);
            translate([0, barb_transition_length + handle_length, - outside_height/2 + thickness/2]){
                encoder_cut();
            }
        }
    }
}

module lid(){
    difference(){
        union(){
        intersection(){
            union(){
                main_body(barb_transition_length, handle_length, outside_height, outside_width);
                rotate([0,180,0]){
                    end_lid_mount(drill=3.5, through=true);
                    mirror([1,0,0])
                    end_lid_mount(drill=3.5, through=true);
                }
            }
            translate([0, 0, -handle_chamfer/2 + outside_height/2])
                cube([100, 300, handle_chamfer], true);
        }
        translate([0, 0, outside_height/2 - lip_height/2 - handle_chamfer])
        intersection(){
                main_body_inset(inset = thickness * 2 + 1);
                cube([100, 300, lip_height], true);
            }
        }
        
        inner_lid_drill(drill=3.5);
        mirror([1,0,0])
        inner_lid_drill(drill=3.5);
        
        //inner screw recess
        translate([0,0,outside_height - screw_recess])
        union(){
            inner_lid_drill(drill=screw_head_dia);
            mirror([1,0,0])
            inner_lid_drill(drill=screw_head_dia);
        }
        
        //end screw recess
        translate([outside_width/2 - end_standoff_dia/2,barb_transition_length + handle_length + end_standoff_dia, outside_height - screw_recess])
            cylinder(d = screw_head_dia, h=outside_height, center=true);
        translate([-outside_width/2 + end_standoff_dia/2, barb_transition_length + handle_length + end_standoff_dia, outside_height - screw_recess])
            cylinder(d = screw_head_dia, h=outside_height, center=true);
        
        //Cutout for DC jack
        translate([0, barb_transition_length + handle_length + thickness, -13/2 + thickness + 7 + 1.6])
            cube([9 + 2, thickness * 10, 11 + 1], true);
        
        inner_lid_mount(cutout = true, clearance=0.5);
        mirror([1, 0, 0]) inner_lid_mount(cutout = true, clearance=0.5);
        main_body_inset(inset = handle_chamfer + thickness * 4, height_inset = thickness * 2);
        
    }
}

lid();
*handle();


